@@ -69,20 +69,12 @@
 #include <locale.h>
 #include <time.h>
 
-#ifdef WITH_CLUTTER
 #include <clutter/clutter.h>
 #include <clutter/x11/clutter-x11.h>
-#endif
 
 #ifdef HAVE_INTROSPECTION
 #include <girepository.h>
-#include "compositor/mutter/mutter-plugin-manager.h"
-#endif
-
-#include <X11/cursorfont.h>
-
-#ifdef HAVE_XCURSOR
-#include <X11/Xcursor/Xcursor.h>
+#include "compositor/mutter-plugin-manager.h"
 #endif
 
 /**
@@ -184,11 +176,6 @@ meta_print_compilation_info (void)
 #else
   meta_verbose ("Compiled without startup notification\n");
 #endif
-#ifdef HAVE_COMPOSITE_EXTENSIONS
-  meta_verbose ("Compiled with composite extensions\n");
-#else
-  meta_verbose ("Compiled without composite extensions\n");
-#endif
 }
 
 /**
@@ -243,12 +230,7 @@ typedef struct
   gchar *introspect;
 } MetaArguments;
 
-#ifdef HAVE_COMPOSITE_EXTENSIONS
 #define COMPOSITE_OPTS_FLAGS 0
-#else /* HAVE_COMPOSITE_EXTENSIONS */
-/* No compositor, so don't show the arguments in --help */
-#define COMPOSITE_OPTS_FLAGS G_OPTION_FLAG_HIDDEN
-#endif /* HAVE_COMPOSITE_EXTENSIONS */
 
 /**
  * Parses argc and argv and returns the
@@ -322,14 +304,12 @@ meta_parse_options (int *argc, char ***argv,
       N_("Turn compositing off"),
       NULL
     },
-#ifdef WITH_CLUTTER
     {
       "mutter-plugins", 0, 0, G_OPTION_ARG_STRING,
       &my_args.mutter_plugins,
       N_("Comma-separated list of compositor plugins"),
       "PLUGINS"
     },
-#endif
     {
       "no-tab-popup", 0, 0, G_OPTION_ARG_NONE,
       &my_args.no_tab_popup,
@@ -350,15 +330,7 @@ meta_parse_options (int *argc, char ***argv,
 
   ctx = g_option_context_new (NULL);
   g_option_context_add_main_entries (ctx, options, "mutter");
-  
-#ifdef WITH_CLUTTER
-  /*
-   * This function is only available in clutter >= 0.8.2
-   */
-#if CLUTTER_CHECK_VERSION(0,8,2)
   g_option_context_add_group (ctx, clutter_get_option_group_without_init ());
-#endif
-#endif
 
   if (!g_option_context_parse (ctx, argc, argv, &error))
     {
@@ -371,8 +343,6 @@ meta_parse_options (int *argc, char ***argv,
   return ctx;
 }
 
-
-#ifdef WITH_CLUTTER
 /* Mutter is responsible for pulling events off the X queue, so Clutter
  * doesn't need (and shouldn't) run its normal event source which polls
  * the X fd, but we do have to deal with dispatching events that accumulate
@@ -425,19 +395,15 @@ meta_clutter_init (GOptionContext *ctx, int *argc, char ***argv)
   
   if (CLUTTER_INIT_SUCCESS == clutter_init (argc, argv))
     {
-      meta_compositor_can_use_clutter__ = 1;
-
       GSource *source = g_source_new (&event_funcs, sizeof (GSource));
       g_source_attach (source, NULL);
       g_source_unref (source);
     }
   else
     {
-      g_message ("Unable to initialize Clutter.\n");
-      meta_compositor_can_use_clutter__ = 0;
+	  meta_fatal ("Unable to initialize Clutter.\n");
     }
 }
-#endif
 
 /**
  * Selects which display Mutter should use. It first tries to use
@@ -461,8 +427,11 @@ meta_select_display (gchar *display_name)
 static void
 meta_finalize (void)
 {
-  meta_display_close (meta_get_display (),
-                      CurrentTime); /* I doubt correct timestamps matter here */
+  MetaDisplay *display = meta_get_display ();
+
+  if (display)
+    meta_display_close (display,
+                        CurrentTime); /* I doubt correct timestamps matter here */
 
   meta_session_shutdown ();
 }
@@ -587,7 +556,6 @@ main (int argc, char **argv)
   /* Parse command line arguments.*/
   ctx = meta_parse_options (&argc, &argv, &meta_args);
 
-#ifdef WITH_CLUTTER
   /* This must come before the introspect below, so we load all the plugins
    * in order to get their get_type functions.
    */
@@ -609,7 +577,6 @@ main (int argc, char **argv)
       g_slist_free(plugins_list);
       g_strfreev (plugins);
     }
-#endif /* WITH_CLUTTER */
 
 #ifdef HAVE_INTROSPECTION
   g_irepository_prepend_search_path (MUTTER_PKGLIBDIR);
@@ -654,43 +621,11 @@ main (int argc, char **argv)
   
   meta_ui_init (&argc, &argv);  
 
-#ifdef WITH_CLUTTER
   /*
    * Clutter can only be initialized after the UI.
    */
   meta_clutter_init (ctx, &argc, &argv);
 
-  {
-    /*
-     * Set busy cursor as soon as we can (at this there is no MetaDisplay or
-     * MetaScreen object, so we have to do this the xlib way.
-     *
-     * Metacity will reset the cursor as soon as it creates the default
-     * MetaScreen object. Plugins might want to further control the cursor
-     * in their constructor.
-     */
-    Display *xdpy = meta_ui_get_display ();
-    Cursor   xc;
-
-#ifdef HAVE_XCURSOR
-    /*
-     * Obviously, this is moblin specific; we want the correct theme
-     * applied from the start, and we cannot query the prefs yet, but the
-     * prefs take a significant amount of time to query, and we do not want
-     * the uggly X watch cursor there at all.
-     */
-    XcursorSetTheme (xdpy, "moblin");
-    XcursorSetDefaultSize (xdpy, 24);
-#endif
-
-    xc = XCreateFontCursor (xdpy, XC_watch);
-
-    XDefineCursor (xdpy, RootWindow (xdpy, 0), xc);
-    XSync (xdpy,False);
-    XFreeCursor (xdpy, xc);
-  }
-#endif
-
   g_option_context_free (ctx);
 
   /* must be after UI init so we can override GDK handlers */
@@ -698,7 +633,6 @@ main (int argc, char **argv)
 
   /* Load prefs */
   meta_prefs_init ();
-
   meta_prefs_add_listener (prefs_changed_callback, NULL);
 
 
@@ -789,17 +723,7 @@ main (int argc, char **argv)
 
   if (!meta_display_open ())
     meta_exit (META_EXIT_ERROR);
-
-  /*
-   * This seems the first time we can set up the cursor properly. The display
-   * seems to set up the theme at the point it opens, but the cursor size
-   * for individual screens seems wrong unless we update it here.
-   *
-   * TODO -- investigate further to avoid unnecessary X round trips.
-   */
-  meta_display_set_cursor_theme (meta_prefs_get_cursor_theme (),
-                                 meta_prefs_get_cursor_size ());
-
+  
   g_main_loop_run (meta_main_loop);
 
   meta_finalize ();
