@@ -39,6 +39,8 @@
 #include "xprops.h"
 #include "compositor.h"
 #include "alttabhandlerdefault.h"
+#include "mutter-marshal.h"
+#include "mutter-enum-types.h"
 
 #ifdef HAVE_SOLARIS_XINERAMA
 #include <X11/extensions/xinerama.h>
@@ -80,6 +82,10 @@ enum
 {
   RESTACKED,
   TOGGLE_RECORDING,
+  WORKSPACE_ADDED,
+  WORKSPACE_REMOVED,
+  WORKSPACE_SWITCHED,
+  STARTUP_SEQUENCE_CHANGED,
 
   LAST_SIGNAL
 };
@@ -159,6 +165,50 @@ meta_screen_class_init (MetaScreenClass *klass)
                             1, G_MAXINT, 1,
                             G_PARAM_READABLE);
 
+  screen_signals[WORKSPACE_ADDED] =
+    g_signal_new ("workspace-added",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__INT,
+                  G_TYPE_NONE,
+                  1,
+                  G_TYPE_INT);
+
+  screen_signals[WORKSPACE_REMOVED] =
+    g_signal_new ("workspace-removed",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__INT,
+                  G_TYPE_NONE,
+                  1,
+                  G_TYPE_INT);
+
+  screen_signals[WORKSPACE_SWITCHED] =
+    g_signal_new ("workspace-switched",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL, NULL,
+                  _mutter_marshal_VOID__INT_INT_ENUM,
+                  G_TYPE_NONE,
+                  3,
+                  G_TYPE_INT,
+                  G_TYPE_INT,
+                  MUTTER_TYPE_MOTION_DIRECTION);
+
+  screen_signals[STARTUP_SEQUENCE_CHANGED] =
+    g_signal_new ("startup-sequence-changed",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__POINTER,
+                  G_TYPE_NONE, 1, G_TYPE_POINTER);
+
   screen_signals[TOGGLE_RECORDING] =
     g_signal_new ("toggle-recording",
                   G_TYPE_FROM_CLASS (klass),
@@ -249,7 +299,7 @@ set_wm_icon_size_hint (MetaScreen *screen)
 }
 
 static void
-reload_xinerama_infos (MetaScreen *screen)
+reload_monitor_infos (MetaScreen *screen)
 {
   MetaDisplay *display;
 
@@ -269,17 +319,36 @@ reload_xinerama_infos (MetaScreen *screen)
 
   display = screen->display;
   
-  if (screen->xinerama_infos)
-    g_free (screen->xinerama_infos);
+  if (screen->monitor_infos)
+    g_free (screen->monitor_infos);
   
-  screen->xinerama_infos = NULL;
-  screen->n_xinerama_infos = 0;
-  screen->last_xinerama_index = 0;
+  screen->monitor_infos = NULL;
+  screen->n_monitor_infos = 0;
+  screen->last_monitor_index = 0;
 
-  screen->display->xinerama_cache_invalidated = TRUE;
+  screen->display->monitor_cache_invalidated = TRUE;
+
+  if (g_getenv ("MUTTER_DEBUG_XINERAMA"))
+    {
+      meta_topic (META_DEBUG_XINERAMA,
+                  "Pretending a single monitor has two Xinerama screens\n");
+
+      screen->monitor_infos = g_new (MetaMonitorInfo, 2);
+      screen->n_monitor_infos = 2;
+
+      screen->monitor_infos[0].number = 0;
+      screen->monitor_infos[0].rect = screen->rect;
+      screen->monitor_infos[0].rect.width = screen->rect.width / 2;
+
+      screen->monitor_infos[1].number = 1;
+      screen->monitor_infos[1].rect = screen->rect;
+      screen->monitor_infos[1].rect.x = screen->rect.width / 2;
+      screen->monitor_infos[1].rect.width = screen->rect.width / 2;
+    }
   
 #ifdef HAVE_XFREE_XINERAMA
-  if (XineramaIsActive (display->xdisplay))
+  if (screen->n_monitor_infos == 0 &&
+      XineramaIsActive (display->xdisplay))
     {
       XineramaScreenInfo *infos;
       int n_infos;
@@ -294,25 +363,25 @@ reload_xinerama_infos (MetaScreen *screen)
 
       if (n_infos > 0)
         {
-          screen->xinerama_infos = g_new (MetaXineramaScreenInfo, n_infos);
-          screen->n_xinerama_infos = n_infos;
+          screen->monitor_infos = g_new (MetaMonitorInfo, n_infos);
+          screen->n_monitor_infos = n_infos;
           
           i = 0;
           while (i < n_infos)
             {
-              screen->xinerama_infos[i].number = infos[i].screen_number;
-              screen->xinerama_infos[i].rect.x = infos[i].x_org;
-              screen->xinerama_infos[i].rect.y = infos[i].y_org;
-              screen->xinerama_infos[i].rect.width = infos[i].width;
-              screen->xinerama_infos[i].rect.height = infos[i].height;
+              screen->monitor_infos[i].number = infos[i].screen_number;
+              screen->monitor_infos[i].rect.x = infos[i].x_org;
+              screen->monitor_infos[i].rect.y = infos[i].y_org;
+              screen->monitor_infos[i].rect.width = infos[i].width;
+              screen->monitor_infos[i].rect.height = infos[i].height;
 
               meta_topic (META_DEBUG_XINERAMA,
-                          "Xinerama %d is %d,%d %d x %d\n",
-                          screen->xinerama_infos[i].number,
-                          screen->xinerama_infos[i].rect.x,
-                          screen->xinerama_infos[i].rect.y,
-                          screen->xinerama_infos[i].rect.width,
-                          screen->xinerama_infos[i].rect.height);
+                          "Monitor %d is %d,%d %d x %d\n",
+                          screen->monitor_infos[i].number,
+                          screen->monitor_infos[i].rect.x,
+                          screen->monitor_infos[i].rect.y,
+                          screen->monitor_infos[i].rect.width,
+                          screen->monitor_infos[i].rect.height);
               
               ++i;
             }
@@ -320,7 +389,7 @@ reload_xinerama_infos (MetaScreen *screen)
       
       meta_XFree (infos);
     }
-  else
+  else if (screen->n_monitor_infos > 0)
     {
       meta_topic (META_DEBUG_XINERAMA,
                   "No XFree86 Xinerama extension or XFree86 Xinerama inactive on display %s\n",
@@ -333,7 +402,7 @@ reload_xinerama_infos (MetaScreen *screen)
 
 #ifdef HAVE_SOLARIS_XINERAMA
   /* This code from GDK, Copyright (C) 2002 Sun Microsystems */
-  if (screen->n_xinerama_infos == 0 &&
+  if (screen->n_monitor_infos == 0 &&
       XineramaGetState (screen->display->xdisplay,
                         screen->number))
     {
@@ -355,31 +424,31 @@ reload_xinerama_infos (MetaScreen *screen)
 	{
           g_assert (n_monitors > 0);
           
-          screen->xinerama_infos = g_new (MetaXineramaScreenInfo, n_monitors);
-          screen->n_xinerama_infos = n_monitors;
+          screen->monitor_infos = g_new (MetaMonitorInfo, n_monitors);
+          screen->n_monitor_infos = n_monitors;
           
           i = 0;
           while (i < n_monitors)
             {
-              screen->xinerama_infos[i].number = i;
-              screen->xinerama_infos[i].rect.x = monitors[i].x;
-              screen->xinerama_infos[i].rect.y = monitors[i].y;
-              screen->xinerama_infos[i].rect.width = monitors[i].width;
-              screen->xinerama_infos[i].rect.height = monitors[i].height;
+              screen->monitor_infos[i].number = i;
+              screen->monitor_infos[i].rect.x = monitors[i].x;
+              screen->monitor_infos[i].rect.y = monitors[i].y;
+              screen->monitor_infos[i].rect.width = monitors[i].width;
+              screen->monitor_infos[i].rect.height = monitors[i].height;
 
               meta_topic (META_DEBUG_XINERAMA,
-                          "Xinerama %d is %d,%d %d x %d\n",
-                          screen->xinerama_infos[i].number,
-                          screen->xinerama_infos[i].rect.x,
-                          screen->xinerama_infos[i].rect.y,
-                          screen->xinerama_infos[i].rect.width,
-                          screen->xinerama_infos[i].rect.height);              
+                          "Monitor %d is %d,%d %d x %d\n",
+                          screen->monitor_infos[i].number,
+                          screen->monitor_infos[i].rect.x,
+                          screen->monitor_infos[i].rect.y,
+                          screen->monitor_infos[i].rect.width,
+                          screen->monitor_infos[i].rect.height);              
               
               ++i;
             }
 	}
     }
-  else if (screen->n_xinerama_infos == 0)
+  else if (screen->n_monitor_infos == 0)
     {
       meta_topic (META_DEBUG_XINERAMA,
                   "No Solaris Xinerama extension or Solaris Xinerama inactive on display %s\n",
@@ -394,40 +463,20 @@ reload_xinerama_infos (MetaScreen *screen)
   /* If no Xinerama, fill in the single screen info so
    * we can use the field unconditionally
    */
-  if (screen->n_xinerama_infos == 0)
+  if (screen->n_monitor_infos == 0)
     {
-      if (g_getenv ("MUTTER_DEBUG_XINERAMA"))
-        {
-          meta_topic (META_DEBUG_XINERAMA,
-                      "Pretending a single monitor has two Xinerama screens\n");
+      meta_topic (META_DEBUG_XINERAMA,
+                  "No Xinerama screens, using default screen info\n");
           
-          screen->xinerama_infos = g_new (MetaXineramaScreenInfo, 2);
-          screen->n_xinerama_infos = 2;
+      screen->monitor_infos = g_new (MetaMonitorInfo, 1);
+      screen->n_monitor_infos = 1;
           
-          screen->xinerama_infos[0].number = 0;
-          screen->xinerama_infos[0].rect = screen->rect;
-          screen->xinerama_infos[0].rect.width = screen->rect.width / 2;
-
-          screen->xinerama_infos[1].number = 1;
-          screen->xinerama_infos[1].rect = screen->rect;
-          screen->xinerama_infos[1].rect.x = screen->rect.width / 2;
-          screen->xinerama_infos[1].rect.width = screen->rect.width / 2;
-        }
-      else
-        {
-          meta_topic (META_DEBUG_XINERAMA,
-                      "No Xinerama screens, using default screen info\n");
-          
-          screen->xinerama_infos = g_new (MetaXineramaScreenInfo, 1);
-          screen->n_xinerama_infos = 1;
-          
-          screen->xinerama_infos[0].number = 0;
-          screen->xinerama_infos[0].rect = screen->rect;
-        }
+      screen->monitor_infos[0].number = 0;
+      screen->monitor_infos[0].rect = screen->rect;
     }
 
-  g_assert (screen->n_xinerama_infos > 0);
-  g_assert (screen->xinerama_infos != NULL);
+  g_assert (screen->n_monitor_infos > 0);
+  g_assert (screen->monitor_infos != NULL);
 }
 
 /* The guard window allows us to leave minimized windows mapped so
@@ -439,11 +488,15 @@ create_guard_window (Display *xdisplay, MetaScreen *screen)
 {
   XSetWindowAttributes attributes;
   Window guard_window;
+  gulong create_serial;
   
   attributes.event_mask = NoEventMask;
   attributes.override_redirect = True;
   attributes.background_pixel = BlackPixel (xdisplay, screen->number);
 
+  /* We have to call record_add() after we have the new window ID,
+   * so save the serial for the CreateWindow request until then */
+  create_serial = XNextRequest(xdisplay);
   guard_window =
     XCreateWindow (xdisplay,
 		   screen->xroot,
@@ -457,6 +510,13 @@ create_guard_window (Display *xdisplay, MetaScreen *screen)
 		   CopyFromParent, /* visual */
 		   CWEventMask|CWOverrideRedirect|CWBackPixel,
 		   &attributes);
+  meta_stack_tracker_record_add (screen->stack_tracker,
+                                 guard_window,
+                                 create_serial);
+
+  meta_stack_tracker_record_lower (screen->stack_tracker,
+                                   guard_window,
+                                   XNextRequest (xdisplay));
   XLowerWindow (xdisplay, guard_window);
   XMapWindow (xdisplay, guard_window);
   return guard_window;
@@ -593,10 +653,7 @@ meta_screen_new (MetaDisplay *display,
                 LeaveWindowMask | EnterWindowMask |
                 KeyPressMask | KeyReleaseMask |
                 FocusChangeMask | StructureNotifyMask |
-#ifdef HAVE_COMPOSITE_EXTENSIONS
-                ExposureMask |
-#endif
-		attr.your_event_mask);
+                ExposureMask | attr.your_event_mask);
   if (meta_error_trap_pop_with_return (display, FALSE) != Success)
     {
       meta_warning (_("Screen %d on display \"%s\" already has a window manager\n"),
@@ -627,11 +684,9 @@ meta_screen_new (MetaDisplay *display,
   screen->wm_sn_atom = wm_sn_atom;
   screen->wm_sn_timestamp = manager_timestamp;
 
-#ifdef HAVE_COMPOSITE_EXTENSIONS
   screen->wm_cm_selection_window = meta_create_offscreen_window (xdisplay, 
                                                                  xroot, 
                                                                  NoEventMask);
-#endif
   screen->work_area_idle = 0;
 
   screen->active_workspace = NULL;
@@ -672,13 +727,13 @@ meta_screen_new (MetaDisplay *display,
                                      &gc_values);
   }
   
-  screen->xinerama_infos = NULL;
-  screen->n_xinerama_infos = 0;
-  screen->last_xinerama_index = 0;  
+  screen->monitor_infos = NULL;
+  screen->n_monitor_infos = 0;
+  screen->last_monitor_index = 0;  
   
-  reload_xinerama_infos (screen);
+  reload_monitor_infos (screen);
   
-  meta_screen_set_cursor (screen, META_CURSOR_BUSY);
+  meta_screen_set_cursor (screen, META_CURSOR_DEFAULT);
 
   /* Handle creating a no_focus_window for this screen */  
   screen->no_focus_window =
@@ -730,6 +785,7 @@ meta_screen_new (MetaDisplay *display,
   screen->ws_popup = NULL;
   
   screen->stack = meta_stack_new (screen);
+  screen->stack_tracker = meta_stack_tracker_new (screen);
 
   meta_prefs_add_listener (prefs_changed_callback, screen);
 
@@ -807,6 +863,7 @@ meta_screen_free (MetaScreen *screen,
   meta_ui_free (screen->ui);
 
   meta_stack_free (screen->stack);
+  meta_stack_tracker_free (screen->stack_tracker);
 
   meta_error_trap_push_with_return (screen->display);
   XSelectInput (screen->display->xdisplay, screen->xroot, 0);
@@ -833,8 +890,8 @@ meta_screen_free (MetaScreen *screen,
   XFreeGC (screen->display->xdisplay,
            screen->root_xor_gc);
   
-  if (screen->xinerama_infos)
-    g_free (screen->xinerama_infos);
+  if (screen->monitor_infos)
+    g_free (screen->monitor_infos);
   
   g_free (screen->screen_name);
 
@@ -913,6 +970,7 @@ meta_screen_manage_all_windows (MetaScreen *screen)
       MetaWindow *window;
 
       window = meta_window_new_with_attrs (screen->display, info->xwindow, TRUE,
+                                           META_COMP_EFFECT_NONE,
                                            &info->attrs);
     }
   meta_stack_thaw (screen->stack);
@@ -926,7 +984,6 @@ meta_screen_manage_all_windows (MetaScreen *screen)
 void
 meta_screen_composite_all_windows (MetaScreen *screen)
 {
-#ifdef HAVE_COMPOSITE_EXTENSIONS
   MetaDisplay *display;
   GSList *windows, *tmp;
 
@@ -934,15 +991,14 @@ meta_screen_composite_all_windows (MetaScreen *screen)
   if (!display->compositor)
     return;
 
-  windows = meta_display_list_windows (display);
+  windows = meta_display_list_windows (display,
+                                       META_LIST_INCLUDE_OVERRIDE_REDIRECT);
   for (tmp = windows; tmp != NULL; tmp = tmp->next)
     meta_compositor_add_window (display->compositor, tmp->data);
   g_slist_free (windows);
   
-  /* trigger a stack_sync_to_server: */
-  meta_stack_freeze (screen->stack);
-  meta_stack_thaw (screen->stack);
-#endif
+  /* initialize the compositor's view of the stacking order */
+  meta_stack_tracker_sync_stack (screen->stack_tracker);
 }
 
 /**
@@ -1044,6 +1100,15 @@ listify_func (gpointer key, gpointer value, gpointer data)
   *listp = g_slist_prepend (*listp, value);
 }
 
+/**
+ * meta_screen_foreach_window:
+ * @screen: a #MetaScreen
+ * @func: function to call for each window
+ * @data: user data to pass to @func
+ *
+ * Calls the specified function for each window on the screen,
+ * ignoring override-redirect windows.
+ */
 void
 meta_screen_foreach_window (MetaScreen *screen,
                             MetaScreenWindowFunc func,
@@ -1074,7 +1139,7 @@ meta_screen_foreach_window (MetaScreen *screen,
         {
           MetaWindow *window = tmp->data;
 
-          if (window->screen == screen)
+          if (window->screen == screen && !window->override_redirect)
             (* func) (screen, window, data);
         }
       
@@ -1226,6 +1291,7 @@ meta_screen_remove_workspace (MetaScreen *screen, MetaWorkspace *workspace,
   GList         *l;
   MetaWorkspace *neighbour = NULL;
   GList         *next = NULL;
+  int index;
 
   l = screen->workspaces;
   while (l)
@@ -1261,6 +1327,9 @@ meta_screen_remove_workspace (MetaScreen *screen, MetaWorkspace *workspace,
   if (workspace == screen->active_workspace)
     meta_workspace_activate (neighbour, timestamp);
 
+  /* To emit the signal after removing the workspace */
+  index = meta_workspace_index (workspace);
+
   /* This also removes the workspace from the screens list */
   meta_workspace_remove (workspace);
 
@@ -1278,6 +1347,7 @@ meta_screen_remove_workspace (MetaScreen *screen, MetaWorkspace *workspace,
 
   meta_screen_queue_workarea_recalc (screen);
 
+  g_signal_emit (screen, screen_signals[WORKSPACE_REMOVED], 0, index);
   g_object_notify (G_OBJECT (screen), "n-workspaces");
 }
 
@@ -1313,6 +1383,8 @@ meta_screen_append_new_workspace (MetaScreen *screen, gboolean activate,
 
   meta_screen_queue_workarea_recalc (screen);
 
+  g_signal_emit (screen, screen_signals[WORKSPACE_ADDED],
+                 0, meta_workspace_index (w));
   g_object_notify (G_OBJECT (screen), "n-workspaces");
 
   return w;
@@ -1490,7 +1562,8 @@ meta_screen_tab_popup_get_selected (MetaScreen *screen)
 void
 meta_screen_tab_popup_destroy (MetaScreen *screen)
 {
-  g_return_if_fail (screen->tab_handler != NULL);
+  if (!screen->tab_handler)
+    return;
 
   meta_alt_tab_handler_destroy (screen->tab_handler);
   g_object_unref (screen->tab_handler);
@@ -1630,61 +1703,61 @@ meta_screen_get_mouse_window (MetaScreen  *screen,
   return window;
 }
 
-const MetaXineramaScreenInfo*
-meta_screen_get_xinerama_for_rect (MetaScreen    *screen,
-				   MetaRectangle *rect)
+const MetaMonitorInfo*
+meta_screen_get_monitor_for_rect (MetaScreen    *screen,
+                                  MetaRectangle *rect)
 {
   int i;
-  int best_xinerama, xinerama_score;
+  int best_monitor, monitor_score;
 
-  if (screen->n_xinerama_infos == 1)
-    return &screen->xinerama_infos[0];
+  if (screen->n_monitor_infos == 1)
+    return &screen->monitor_infos[0];
 
-  best_xinerama = 0;
-  xinerama_score = 0;
+  best_monitor = 0;
+  monitor_score = 0;
 
-  for (i = 0; i < screen->n_xinerama_infos; i++)
+  for (i = 0; i < screen->n_monitor_infos; i++)
     {
       MetaRectangle dest;
-      if (meta_rectangle_intersect (&screen->xinerama_infos[i].rect,
+      if (meta_rectangle_intersect (&screen->monitor_infos[i].rect,
                                     rect,
                                     &dest))
         {
           int cur = meta_rectangle_area (&dest);
-          if (cur > xinerama_score)
+          if (cur > monitor_score)
             {
-              xinerama_score = cur;
-              best_xinerama = i;
+              monitor_score = cur;
+              best_monitor = i;
             }
         }
     }
 
-  return &screen->xinerama_infos[best_xinerama];
+  return &screen->monitor_infos[best_monitor];
 }
 
-const MetaXineramaScreenInfo*
-meta_screen_get_xinerama_for_window (MetaScreen *screen,
-				     MetaWindow *window)
+const MetaMonitorInfo*
+meta_screen_get_monitor_for_window (MetaScreen *screen,
+                                    MetaWindow *window)
 {
   MetaRectangle window_rect;
   
   meta_window_get_outer_rect (window, &window_rect);
 
-  return meta_screen_get_xinerama_for_rect (screen, &window_rect);
+  return meta_screen_get_monitor_for_rect (screen, &window_rect);
 }
 
-const MetaXineramaScreenInfo* 
-meta_screen_get_xinerama_neighbor (MetaScreen         *screen,
-                                   int                 which_xinerama,
-                                   MetaScreenDirection direction)
+const MetaMonitorInfo* 
+meta_screen_get_monitor_neighbor (MetaScreen         *screen,
+                                  int                 which_monitor,
+                                  MetaScreenDirection direction)
 {
-  MetaXineramaScreenInfo* input = screen->xinerama_infos + which_xinerama;
-  MetaXineramaScreenInfo* current;
+  MetaMonitorInfo* input = screen->monitor_infos + which_monitor;
+  MetaMonitorInfo* current;
   int i;
 
-  for (i = 0; i < screen->n_xinerama_infos; i++)
+  for (i = 0; i < screen->n_monitor_infos; i++)
     {
-      current = screen->xinerama_infos + i;
+      current = screen->monitor_infos + i;
 
       if ((direction == META_SCREEN_RIGHT && 
            current->rect.x == input->rect.x + input->rect.width &&
@@ -1707,110 +1780,110 @@ meta_screen_get_xinerama_neighbor (MetaScreen         *screen,
 }
 
 void
-meta_screen_get_natural_xinerama_list (MetaScreen *screen,
-                                       int**       xineramas_list,
-                                       int*        n_xineramas)
+meta_screen_get_natural_monitor_list (MetaScreen *screen,
+                                      int**       monitors_list,
+                                      int*        n_monitors)
 {
-  const MetaXineramaScreenInfo* current;
-  const MetaXineramaScreenInfo* tmp;
-  GQueue* xinerama_queue;
+  const MetaMonitorInfo* current;
+  const MetaMonitorInfo* tmp;
+  GQueue* monitor_queue;
   int* visited;
   int cur = 0;
   int i;
 
-  *n_xineramas = screen->n_xinerama_infos;
-  *xineramas_list = g_new (int, screen->n_xinerama_infos);
+  *n_monitors = screen->n_monitor_infos;
+  *monitors_list = g_new (int, screen->n_monitor_infos);
 
-  /* we calculate a natural ordering by which to choose xineramas for
-   * window placement.  We start at the current xinerama, and perform
-   * a breadth-first search of the xineramas starting from that
-   * xinerama.  We choose preferentially left, then right, then down,
+  /* we calculate a natural ordering by which to choose monitors for
+   * window placement.  We start at the current monitor, and perform
+   * a breadth-first search of the monitors starting from that
+   * monitor.  We choose preferentially left, then right, then down,
    * then up.  The visitation order produced by this traversal is the
-   * natural xinerama ordering.
+   * natural monitor ordering.
    */
 
-  visited = g_new (int, screen->n_xinerama_infos);
-  for (i = 0; i < screen->n_xinerama_infos; i++)
+  visited = g_new (int, screen->n_monitor_infos);
+  for (i = 0; i < screen->n_monitor_infos; i++)
     {
       visited[i] = FALSE;
     }
 
-  current = meta_screen_get_current_xinerama (screen);
-  xinerama_queue = g_queue_new ();
-  g_queue_push_tail (xinerama_queue, (gpointer) current);
+  current = meta_screen_get_current_monitor (screen);
+  monitor_queue = g_queue_new ();
+  g_queue_push_tail (monitor_queue, (gpointer) current);
   visited[current->number] = TRUE;
 
-  while (!g_queue_is_empty (xinerama_queue))
+  while (!g_queue_is_empty (monitor_queue))
     {
-      current = (const MetaXineramaScreenInfo*) 
-        g_queue_pop_head (xinerama_queue);
+      current = (const MetaMonitorInfo*) 
+        g_queue_pop_head (monitor_queue);
 
-      (*xineramas_list)[cur++] = current->number;
+      (*monitors_list)[cur++] = current->number;
 
       /* enqueue each of the directions */
-      tmp = meta_screen_get_xinerama_neighbor (screen,
-                                               current->number,
-                                               META_SCREEN_LEFT);
+      tmp = meta_screen_get_monitor_neighbor (screen,
+                                              current->number,
+                                              META_SCREEN_LEFT);
       if (tmp && !visited[tmp->number])
         {
-          g_queue_push_tail (xinerama_queue,
-                             (MetaXineramaScreenInfo*) tmp);
+          g_queue_push_tail (monitor_queue,
+                             (MetaMonitorInfo*) tmp);
           visited[tmp->number] = TRUE;
         }
-      tmp = meta_screen_get_xinerama_neighbor (screen,
-                                               current->number,
-                                               META_SCREEN_RIGHT);
+      tmp = meta_screen_get_monitor_neighbor (screen,
+                                              current->number,
+                                              META_SCREEN_RIGHT);
       if (tmp && !visited[tmp->number])
         {
-          g_queue_push_tail (xinerama_queue,
-                             (MetaXineramaScreenInfo*) tmp);
+          g_queue_push_tail (monitor_queue,
+                             (MetaMonitorInfo*) tmp);
           visited[tmp->number] = TRUE;
         }
-      tmp = meta_screen_get_xinerama_neighbor (screen,
-                                               current->number,
-                                               META_SCREEN_UP);
+      tmp = meta_screen_get_monitor_neighbor (screen,
+                                              current->number,
+                                              META_SCREEN_UP);
       if (tmp && !visited[tmp->number])
         {
-          g_queue_push_tail (xinerama_queue,
-                             (MetaXineramaScreenInfo*) tmp);
+          g_queue_push_tail (monitor_queue,
+                             (MetaMonitorInfo*) tmp);
           visited[tmp->number] = TRUE;
         }
-      tmp = meta_screen_get_xinerama_neighbor (screen,
-                                               current->number,
-                                               META_SCREEN_DOWN);
+      tmp = meta_screen_get_monitor_neighbor (screen,
+                                              current->number,
+                                              META_SCREEN_DOWN);
       if (tmp && !visited[tmp->number])
         {
-          g_queue_push_tail (xinerama_queue,
-                             (MetaXineramaScreenInfo*) tmp);
+          g_queue_push_tail (monitor_queue,
+                             (MetaMonitorInfo*) tmp);
           visited[tmp->number] = TRUE;
         }
     }
 
-  /* in case we somehow missed some set of xineramas, go through the
-   * visited list and add in any xineramas that were missed
+  /* in case we somehow missed some set of monitors, go through the
+   * visited list and add in any monitors that were missed
    */
-  for (i = 0; i < screen->n_xinerama_infos; i++)
+  for (i = 0; i < screen->n_monitor_infos; i++)
     {
       if (visited[i] == FALSE)
         {
-          (*xineramas_list)[cur++] = i;
+          (*monitors_list)[cur++] = i;
         }
     }
 
   g_free (visited);
-  g_queue_free (xinerama_queue);
+  g_queue_free (monitor_queue);
 }
 
-const MetaXineramaScreenInfo*
-meta_screen_get_current_xinerama (MetaScreen *screen)
+const MetaMonitorInfo*
+meta_screen_get_current_monitor (MetaScreen *screen)
 {
-  if (screen->n_xinerama_infos == 1)
-    return &screen->xinerama_infos[0];
+  if (screen->n_monitor_infos == 1)
+    return &screen->monitor_infos[0];
   
   /* Sadly, we have to do it this way. Yuck.
    */
   
-  if (screen->display->xinerama_cache_invalidated)
+  if (screen->display->monitor_cache_invalidated)
     {
       Window root_return, child_return;
       int win_x_return, win_y_return;
@@ -1818,7 +1891,7 @@ meta_screen_get_current_xinerama (MetaScreen *screen)
       int i;
       MetaRectangle pointer_position;
       
-      screen->display->xinerama_cache_invalidated = FALSE;
+      screen->display->monitor_cache_invalidated = FALSE;
       
       pointer_position.width = pointer_position.height = 1;
       XQueryPointer (screen->display->xdisplay,
@@ -1831,23 +1904,59 @@ meta_screen_get_current_xinerama (MetaScreen *screen)
                      &win_y_return,
                      &mask_return);
 
-      screen->last_xinerama_index = 0;
-      for (i = 0; i < screen->n_xinerama_infos; i++)
+      screen->last_monitor_index = 0;
+      for (i = 0; i < screen->n_monitor_infos; i++)
         {
-          if (meta_rectangle_contains_rect (&screen->xinerama_infos[i].rect,
+          if (meta_rectangle_contains_rect (&screen->monitor_infos[i].rect,
                                             &pointer_position))
             {
-              screen->last_xinerama_index = i;
+              screen->last_monitor_index = i;
               break;
             }
         }
       
       meta_topic (META_DEBUG_XINERAMA,
-                  "Rechecked current Xinerama, now %d\n",
-                  screen->last_xinerama_index);
+                  "Rechecked current monitor, now %d\n",
+                  screen->last_monitor_index);
     }
 
-  return &screen->xinerama_infos[screen->last_xinerama_index];
+  return &screen->monitor_infos[screen->last_monitor_index];
+}
+
+/**
+ * meta_screen_get_n_monitors:
+ * @screen: a #MetaScreen
+ *
+ * Gets the number of monitors that are joined together to form @screen.
+ *
+ * Return value: the number of monitors
+ */
+int
+meta_screen_get_n_monitors (MetaScreen *screen)
+{
+  g_return_val_if_fail (META_IS_SCREEN (screen), 0);
+
+  return screen->n_monitor_infos;
+}
+
+/**
+ * meta_screen_get_monitor_geometry:
+ * @screen: a #MetaScreen
+ * @monitor: the monitor number
+ * @geometry: (out): location to store the monitor geometry
+ *
+ * Stores the location and size of the indicated monitor in @geometry.
+ */
+void
+meta_screen_get_monitor_geometry (MetaScreen    *screen,
+                                  int            monitor,
+                                  MetaRectangle *geometry)
+{
+  g_return_if_fail (META_IS_SCREEN (screen));
+  g_return_if_fail (monitor >= 0 && monitor < screen->n_monitor_infos);
+  g_return_if_fail (geometry != NULL);
+
+  *geometry = screen->monitor_infos[monitor].rect;
 }
 
 #define _NET_WM_ORIENTATION_HORZ 0
@@ -2072,7 +2181,7 @@ set_work_area_hint (MetaScreen *screen)
 
       if (workspace->screen == screen)
         {
-          meta_workspace_get_work_area_all_xineramas (workspace, &area);
+          meta_workspace_get_work_area_all_monitors (workspace, &area);
           tmp[0] = area.x;
           tmp[1] = area.y;
           tmp[2] = area.width;
@@ -2440,7 +2549,7 @@ meta_screen_resize (MetaScreen *screen,
   screen->rect.width = width;
   screen->rect.height = height;
 
-  reload_xinerama_infos (screen);
+  reload_monitor_infos (screen);
   set_desktop_geometry_hint (screen);
   
   if (screen->display->compositor)
@@ -2476,7 +2585,7 @@ queue_windows_showing (MetaScreen *screen)
    * active_workspace's window list, because the active_workspace's
    * window list may not contain the on_all_workspace windows.
    */
-  windows = meta_display_list_windows (screen->display);
+  windows = meta_display_list_windows (screen->display, META_LIST_DEFAULT);
 
   tmp = windows;
   while (tmp != NULL)
@@ -2599,9 +2708,9 @@ add_sequence (MetaScreen        *screen,
    * to compute exactly when we may next time out
    */
   if (screen->startup_sequence_timeout == 0)
-    screen->startup_sequence_timeout = g_timeout_add (1000,
-                                                      startup_sequence_timeout,
-                                                      screen);
+    screen->startup_sequence_timeout = g_timeout_add_seconds (1,
+                                                              startup_sequence_timeout,
+                                                              screen);
 
   update_startup_feedback (screen);
 }
@@ -2616,8 +2725,7 @@ remove_sequence (MetaScreen        *screen,
   
   screen->startup_sequences = g_slist_remove (screen->startup_sequences,
                                               sequence);
-  sn_startup_sequence_unref (sequence);
-  
+
   if (screen->startup_sequences == NULL &&
       screen->startup_sequence_timeout != 0)
     {
@@ -2626,6 +2734,8 @@ remove_sequence (MetaScreen        *screen,
     }
 
   update_startup_feedback (screen);
+
+  sn_startup_sequence_unref (sequence);
 }
 
 typedef struct
@@ -2716,7 +2826,9 @@ meta_screen_sn_event (SnMonitorEvent *event,
   screen = user_data;
 
   sequence = sn_monitor_event_get_startup_sequence (event);
-  
+
+  sn_startup_sequence_ref (sequence);
+
   switch (sn_monitor_event_get_type (event))
     {
     case SN_MONITOR_EVENT_INITIATED:
@@ -2755,6 +2867,22 @@ meta_screen_sn_event (SnMonitorEvent *event,
                   sn_startup_sequence_get_id (sequence));
       break;
     }
+
+  g_signal_emit (G_OBJECT (screen), screen_signals[STARTUP_SEQUENCE_CHANGED], 0, sequence);
+
+  sn_startup_sequence_unref (sequence);
+}
+
+/**
+ * meta_screen_get_startup_sequences:
+ * @screen:
+ *
+ * Return value: (transfer none): Currently active #SnStartupSequence items
+ */
+GSList *
+meta_screen_get_startup_sequences (MetaScreen *screen)
+{
+  return screen->startup_sequences;
 }
 #endif
 
@@ -2945,7 +3073,6 @@ meta_screen_set_compositor_data (MetaScreen *screen,
   screen->compositor_data = compositor;
 }
 
-#ifdef HAVE_COMPOSITE_EXTENSIONS
 void
 meta_screen_set_cm_selection (MetaScreen *screen)
 {
@@ -2969,7 +3096,6 @@ meta_screen_unset_cm_selection (MetaScreen *screen)
   a = XInternAtom (screen->display->xdisplay, selection, FALSE);
   XSetSelectionOwner (screen->display->xdisplay, a, None, CurrentTime);
 }
-#endif /* HAVE_COMPOSITE_EXTENSIONS */
 
 GList *
 meta_screen_get_workspaces (MetaScreen *screen)
@@ -2988,6 +3114,11 @@ meta_screen_get_active_workspace_index (MetaScreen *screen)
   return meta_workspace_index (active);
 }
 
+/**
+ * meta_screen_get_active_workspace:
+ *
+ * Returns: (transfer none): The current workspace
+ */
 MetaWorkspace *
 meta_screen_get_active_workspace (MetaScreen *screen)
 {
@@ -2999,3 +3130,14 @@ meta_screen_restacked (MetaScreen *screen)
 {
   g_signal_emit (screen, screen_signals[RESTACKED], 0);
 }
+
+void
+meta_screen_workspace_switched (MetaScreen         *screen,
+                                int                 from,
+                                int                 to,
+                                MetaMotionDirection direction)
+{
+  g_signal_emit (screen, screen_signals[WORKSPACE_SWITCHED], 0,
+                 from, to, direction);
+}
+
